/*

                Cۏ̎WƋL^

	rtc4543.c

	Copyrights 2010,  Yoshi Hiroshi.

 */

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <string.h>
#include <util/delay.h>
#include "ecr1264.h"
#include "rtc4543.h"

volatile char timerSetMode = 0;
unsigned char sw1count = 0;
unsigned char sw2count = 0;
char sw1last = 99;
char sw2last = 99;
volatile char swChanged = 0;
volatile int counter = 0;

//--------------------------------------------------
//
//
//
//--------------------------------------------------
void initRtc()
{
    RTC_DDR |= _BV(RTC_CLK) | _BV(RTC_CE) | _BV(RTC_WR);
    RTC_DISELECT;
    RTC_CLK_LOW;
    timerSetMode = 0;
    counter = 0;
}

//--------------------------------------------------
//
//
//
//--------------------------------------------------
void getDatetime(unsigned char dateTime[7])
{
    unsigned char rcvBuff = 0;
    RTC_SET_READ;
    RTC_SELECT;
    for(unsigned char dInx = 0; dInx < 7; dInx++) {
        for(unsigned char bInx = 0; bInx  < 8; bInx++) {
            RTC_CLK_HIGH;
            RTC_CLK_HIGH;
            rcvBuff = rcvBuff >> 1;
            RTC_CLK_LOW;
            if(RTC_GET_DATA) rcvBuff |= 0x80;
            if((dInx == 3) && (bInx == 3)) bInx = 8;
        }
        dateTime[dInx] = rcvBuff;
    }
    RTC_DISELECT;
//    dateTime[1] &= 0x7F;    // min
    dateTime[2] &= 0x3F;    // hour
    dateTime[3] &= 0x07;
    dateTime[4] &= 0x3F;    // day
    dateTime[5] &= 0x1F;    // month
//    dateTime[6] &= 0x7F;    // year
}

//--------------------------------------------------
//
//
//
//--------------------------------------------------
void putDatetime(unsigned char dateTime[7])
{
    unsigned char rcvBuff = 0;
    RTC_SET_WRITE;
    RTC_SELECT;
    for(unsigned char dInx = 0; dInx < 7; dInx++) {
        rcvBuff = dateTime[dInx];
        for(unsigned char bInx = 0; bInx  < 8; bInx++) {
            if(rcvBuff & 0x01) {
                RTC_DATA_HIGH;
            } else {
                RTC_DATA_LOW;
            }
            RTC_CLK_HIGH;
            rcvBuff = rcvBuff >> 1;
            RTC_CLK_LOW;
            if((dInx == 3) && (bInx == 3)) bInx = 8;
        }
    }
    RTC_DISELECT;
}

//--------------------------------------------------
//
//
//
//--------------------------------------------------
char checkSw(char val, char* last, unsigned char* count)
{
    char result = 0;
    if(val != *last) {
        if(val != 0) {          //  0->1 release
            if((*count < 8) && (*count >= 1)) {
                result = 1;         // short
            }
        } else {                // 1->0 push
            *count = 0;
        }
        *last = val;
    } else {
        if((val == 0) && (*count == 8)) {
            result = 2;         // long
        }
        if(*count < 200) (*count)++;
    }
    return result;
}

//--------------------------------------------------
//
//
//
//--------------------------------------------------
void showTimesetPanel(unsigned char dateTime[7])
{
    printf("ER\n\r");
    //printf("AP0,0\rAW'dd  hh  mm'\r");
    printf_P(PSTR("SP2,0\rSW'3B7E396F244E405F446A'\r"));    //   ̐ݒ
    printf_P(PSTR("SP6,2\rSW'3B7E'\r"));        //   
    printf_P(PSTR("SP9,2\rSW'4A2C'\r"));        //   

    printf_P(PSTR("AP9,2\rAW'%2X'\r"), dateTime[2]);    // hour
    printf_P(PSTR("AP15,2\rAW'%2x'\r"), dateTime[1]);    // minute

    switch(timerSetMode) {
    case 4:
        printf_P(PSTR("AP9,1\rAW'__'\r"));    // hour
        printf_P(PSTR("AP9,3\rAW'~~'\r"));    // hour
        break;
    case 5:
        printf_P(PSTR("AP15,1\rAW'__'\r"));    // day
        printf_P(PSTR("AP15,3\rAW'~~'\r"));    // day
        break;
    }
    printf("DP\r");
    _delay_ms(10);
}

//--------------------------------------------------
//
//
//
//--------------------------------------------------
void showDatesetPanel(unsigned char dateTime[7])
{
    printf_P(PSTR("ER\n\r"));
    //printf("AP0,0\rAW'dd  hh  mm'\r");
    printf_P(PSTR("SP2,0\rSW'467C4955244E405F446A'\r"));    //   t̐ݒ
    printf_P(PSTR("SP3,2\rSW'472F'\r"));        //   N
    printf_P(PSTR("SP6,2\rSW'376E'\r"));        //   
    printf_P(PSTR("SP9,2\rSW'467C'\r"));        //   

    printf_P(PSTR("AP1,2\rAW'20%02x'\r"), dateTime[6]);     // year
    printf_P(PSTR("AP9,2\rAW'%2x'\r"), dateTime[5]);        // month
    printf_P(PSTR("AP15,2\rAW'%2x'\r"), dateTime[4]);       // day

    switch(timerSetMode) {
    case 1:
        printf_P(PSTR("AP3,1\rAW'__'\r"));    // year
        printf_P(PSTR("AP3,3\rAW'~~'\r"));    // year
        break;
    case 2:
        printf_P(PSTR("AP9,1\rAW'__'\r"));    // month
        printf_P(PSTR("AP9,3\rAW'~~'\r"));    // month
        break;
    case 3:
        printf_P(PSTR("AP15,1\rAW'__'\r"));    // day
        printf_P(PSTR("AP15,3\rAW'~~'\r"));    // day
        break;
    }
    printf_P(PSTR("DP\r"));
    _delay_ms(10);
}

//--------------------------------------------------
//
//
//
//--------------------------------------------------
char modData(unsigned char* data, char max, char min, char updown,
             char escape, char currMode, char nextMode)
{
    char result = currMode;
    if(escape) {
        result = nextMode;
    } else {
        int work = *data;
        work = (work >> 4) * 10 + (work & 0x0F);
        if(updown) {
            work++;
        } else {
            work--;
        }
        if(work > max) work = min;
        if(work < min) work = max;
        *data = ((work / 10) << 4) | work % 10;
    }

    return result;
}

//--------------------------------------------------
//
//
//
//--------------------------------------------------
void dateTimeSet(unsigned char dateTime[7])
{
    static unsigned char dateTimeSave[7];
    char sw1;
    char sw2;
    DDRB |= 1;
    if(timerSetMode == 0) 
    {
        getDatetime(dateTime);
        memcpy(dateTimeSave, dateTime, 7);
        timerSetMode = 1;
        if(timerSetMode < 4) {
            showDatesetPanel(dateTime);
        } else {
            showTimesetPanel(dateTime);
        }
        swChanged = 0;
    } else {
        sw1 = checkSw(PINC & 0x02, &sw1last, &sw1count);
        sw2 = checkSw(PINC & 0x01, &sw2last, &sw2count);
        if(sw1 || sw2) {
            swChanged = 1;
            if(sw2 == 2) {
                while((PINC & 0x03) != 3);
                if(memcmp(dateTimeSave, dateTime, 7) != 0) putDatetime(dateTime);
                _delay_ms(50);
                timerSetMode = 0;
                timerMode = TIMER_1S_MODE_IDLE;
                timerCounter = 0;
                TIMER2_SET_FAST;
            } else {
                switch(timerSetMode) {
                case 1:     // year
                    timerSetMode = modData(&dateTime[6], 99, 0, (sw2 == 1), (sw1 == 2), timerSetMode, 2);
                    break;
                case 2:     // month
                    timerSetMode = modData(&dateTime[5], 12, 1, (sw2 == 1), (sw1 == 2), timerSetMode, 3);
                    break;
                case 3:     // day
                    timerSetMode = modData(&dateTime[4], 31, 1, (sw2 == 1), (sw1 == 2), timerSetMode, 4);
                    break;
                case 4:     // hour
                    timerSetMode = modData(&dateTime[2], 23, 0, (sw2 == 1), (sw1 == 2), timerSetMode, 5);
                    break;
                case 5:     // min
                    timerSetMode = modData(&dateTime[1], 59, 0, (sw2 == 1), (sw1 == 2), timerSetMode, 1);
                    break;
                case 0:
                default:
                    break;
                }
            }
        }
    }

    if(swChanged) {
        counter++;
        if(counter > 8)
        {
            if(timerSetMode < 4) {
                showDatesetPanel(dateTime);
            } else {
                showTimesetPanel(dateTime);
            }
            swChanged = 0;
            counter = 0;
        }
    } else {
        counter = 0;
    }
    PORTB &= 0xFE;
}

